@@ -37,6 +37,7 @@
  * Initialization, shutdown, and version-check functions.
  **/
 
+extern gboolean _g_mime_enable_rfc2047_workarounds (void);
 
 GQuark gmime_error_quark;
 
@@ -118,6 +119,7 @@ g_mime_init (guint32 flags)
 	internet_address_mailbox_get_type ();
 	
 	/* register our default mime object types */
+	g_mime_object_type_registry_init ();
 	g_mime_object_register_type ("*", "*", g_mime_part_get_type ());
 	g_mime_object_register_type ("multipart", "*", g_mime_multipart_get_type ());
 	g_mime_object_register_type ("multipart", "encrypted", g_mime_multipart_encrypted_get_type ());
@@ -141,6 +143,7 @@ g_mime_shutdown (void)
 	if (--initialized)
 		return;
 	
+	g_mime_object_type_registry_shutdown ();
 	g_mime_charset_map_shutdown ();
 	g_mime_iconv_shutdown ();
 }
